/* CoreDeployCalculator.java created by root on Mon 20-Mar-2000 */

import com.apple.yellow.foundation.*;
import com.apple.yellow.eocontrol.*;
import com.apple.yellow.eoaccess.*;
import com.apple.yellow.webobjects.*;
import java.math.*;


public class CoreDeployCalculator {

    // Static Variables
    //

    public static final int	INSTANCE_MODE		= 1;
    public static final int	TRANSACTION_MODE	= 2;

    
    // Instance Variables
    //
    private int mode;
    private BigDecimal loopTime;
    private BigDecimal maxWaitTime;
    private BigDecimal initializedRAM;
    private BigDecimal idealInstanceCount;
    private BigDecimal tenRAM;
    private BigDecimal tPD;
    private BigDecimal userPause;
    private BigDecimal visitTransactionCount;
    private BigDecimal sessionTimeOut;
    

    // Constructors
    //
    public CoreDeployCalculator(int pMode) {
        super();

        mode = pMode;

        setLoopTime(new BigDecimal(1.0));
        setMaxWaitTime(new BigDecimal(1.0));
        setInitializedRAM(new BigDecimal(1.0));
        setIdealInstanceCount(new BigDecimal(1.0));
        setTenRAM(new BigDecimal(1.0));
        setUserPause(new BigDecimal(1.0));
        setTPD(new BigDecimal(1));
        setVisitTransactionCount(new BigDecimal(1.0));
        setSessionTimeOut(new BigDecimal(1.0));
    }

    
    // Accessor Methods
    //
    public int mode() {
        return mode;
    }
    public void setMode(int value) {
        mode = value;
    }
    
    public BigDecimal loopTime() {
        return loopTime;
    }
    public void setLoopTime(BigDecimal value) {
        loopTime = value;
    }
    
    public BigDecimal maxWaitTime() {
        return maxWaitTime;
    }
    public void setMaxWaitTime(BigDecimal value) {
        maxWaitTime = value;
    }

    public BigDecimal initializedRAM() {
        return initializedRAM;
    }
    public void setInitializedRAM(BigDecimal value) {
        initializedRAM = value;
    }

    public BigDecimal idealInstanceCount() {
        if( mode == INSTANCE_MODE) idealInstanceCount = tPS().multiply(loopTime());

        if( idealInstanceCount.doubleValue() < 1.0) idealInstanceCount = new BigDecimal(1.0);
        return idealInstanceCount;
    }
    public void setIdealInstanceCount(BigDecimal value) {
        idealInstanceCount = value;
    }

    public BigDecimal tenRAM() {
        return tenRAM;
    }
    public void setTenRAM(BigDecimal value) {
        tenRAM = value;
    }

    public BigDecimal tPD() {
        if( mode == TRANSACTION_MODE)
            tPD = ((tPS().multiply(new BigDecimal(60.0))).multiply(new BigDecimal(60.0))).multiply(new BigDecimal(24.0));
        return tPD;
    }
    public void setTPD(BigDecimal value) {
        tPD = value;
    }

    public BigDecimal userPause() {
        return userPause;
    }
    public void setUserPause(BigDecimal value) {
        userPause = value;
    }

    public BigDecimal visitTransactionCount() {
        return visitTransactionCount;
    }
    public void setVisitTransactionCount(BigDecimal value) {
        visitTransactionCount = value;
    }

    public BigDecimal sessionTimeOut() {
        return sessionTimeOut;
    }
    public void setSessionTimeOut(BigDecimal value) {
        sessionTimeOut = value;
    }

    // Instance Methods
    //
    public BigDecimal maxQueued() {
        BigDecimal bd = new BigDecimal(1.0);

        if (loopTime().doubleValue() > 0.0) {
            bd = maxWaitTime().divide(loopTime(), 2, BigDecimal.ROUND_DOWN);
        }

        return bd;
    }

    public BigDecimal sessionRAM() {
        return (tenRAM().subtract(initializedRAM())).divide(new BigDecimal(10.0), 2, BigDecimal.ROUND_UP);
    }

    public BigDecimal appRAM() {
        return initializedRAM().subtract(sessionRAM());
    }

    public BigDecimal spikeTPD() {
        return maxQueued().multiply(tPD()); // needs to be rounded up
    }

    public BigDecimal tPH() {
        BigDecimal tph;
        
        if( mode == INSTANCE_MODE)
            tph = tPD().divide(new BigDecimal(24.0), 2, BigDecimal.ROUND_UP);
        else
            tph = (tPS().multiply(new BigDecimal(60.0))).multiply(new BigDecimal(60.0));
        
        return tph;
    }

    public BigDecimal spikeTPH() {
        return spikeTPD().divide(new BigDecimal(24.0), 2, BigDecimal.ROUND_DOWN);
    }

    public BigDecimal tPM() {
        BigDecimal tpm;

        if( mode == INSTANCE_MODE)
            tpm = (tPD().divide(new BigDecimal(24.0), 2, BigDecimal.ROUND_UP)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP);
        else
            tpm = tPS().multiply(new BigDecimal(60.0));

        return tpm;
    }

    public BigDecimal spikeTPM() {
        return (spikeTPD().divide(new BigDecimal(24.0), 2, BigDecimal.ROUND_DOWN)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_DOWN);
    }

    public BigDecimal tPS() {
        BigDecimal tps;

        if( mode == INSTANCE_MODE)
            tps = ((tPD().divide(new BigDecimal(24.0), 2, BigDecimal.ROUND_UP)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP);
        else
            tps = idealInstanceCount().divide(loopTime(), 2, BigDecimal.ROUND_UP);

        return tps;
    }

    public BigDecimal spikeTPS() {
        return ((spikeTPD().divide(new BigDecimal(24.0), 2, BigDecimal.ROUND_DOWN)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_DOWN);
    }

    public BigDecimal sessionLength() {
        return (userPause().add(loopTime())).multiply(visitTransactionCount());
    }

    public BigDecimal spikeSessionLength() {
        return (userPause().add(maxWaitTime())).multiply(visitTransactionCount());
    }

    public BigDecimal sPH() {
        return tPH().divide(visitTransactionCount(), 2, BigDecimal.ROUND_UP);
    }

    public BigDecimal spikeSPH() {
        return spikeTPH().divide(visitTransactionCount(), 2, BigDecimal.ROUND_UP);
    }

    public BigDecimal activeSessions() {
        return sPH().multiply(((sessionTimeOut().add(sessionLength())).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP));
    }

    public BigDecimal spikeActiveSessions() {
        return spikeSPH().multiply(((sessionTimeOut().add(spikeSessionLength())).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP)).divide(new BigDecimal(60.0), 2, BigDecimal.ROUND_UP));
    }

    public BigDecimal totalRAM() {
        return (appRAM().multiply(idealInstanceCount())).add(activeSessions().multiply(sessionRAM()));
    }

    public BigDecimal spikeTotalRAM() {
        return (appRAM().multiply(idealInstanceCount())).add(spikeActiveSessions().multiply(sessionRAM()));
    }
}
